#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
usage: plants_update01.py [-h] [-o OUTFILENAME] [-f] [-v] infilename

synopsis:
  parse an XML file; modify an object; write-out the modified object tree.

positional arguments:
  infilename            Input XML file name

optional arguments:
  -h, --help            show this help message and exit
  -o OUTFILENAME, --outfilename OUTFILENAME
                        Name/path of file to write to (default is stdout)
  -f, --force           Force overwrite of output file without asking
  -v, --verbose         Print messages during actions.

examples:
  python plants_update01.py plants_doc.xml
  python plants_update01.py -v plants_doc.xml
"""

import sys
import os
import argparse
import plants01lib as gdslib


def parse(inFileName, silence=False, print_warnings=True):
    gds_collector = gdslib.GdsCollector_()
    parser = None
    doc = gdslib.parsexml_(inFileName, parser)
    rootNode = doc.getroot()
    #rootTag, rootClass = gdslib.get_root_tag(rootNode)
    #if rootClass is None:
    #    rootTag = 'plantCollection'
    #    rootClass = plantCollection
    rootTag = 'plantCollection'
    rootClass = gdslib.plantCollection
    rootObj = rootClass.factory()
    rootObj.build(rootNode, gds_collector_=gds_collector)
    gdslib.CapturedNsmap_, namespacedefs = gdslib.get_required_ns_prefix_defs(
        rootNode)
    if not gdslib.SaveElementTreeNode:
        doc = None
        rootNode = None
    if not silence:
        sys.stdout.write('<?xml version="1.0" ?>\n')
        rootObj.export(
            sys.stdout, 0, name_=rootTag,
            namespacedef_=namespacedefs,
            pretty_print=True)
    if print_warnings and len(gds_collector.get_messages()) > 0:
        separator = ('-' * 50) + '\n'
        sys.stderr.write(separator)
        sys.stderr.write('----- Warnings -- count: {} -----\n'.format(
            len(gds_collector.get_messages()), ))
        gds_collector.write_messages(sys.stderr)
        sys.stderr.write(separator)
    return rootTag, rootObj


def parse_and_update(opts):
    # parse the XML imput file; create the gDS object tree.
    tag, plantCollection = parse(
        opts.infilename,
        silence=(not opts.verbose),
    )
    # retrieve the `fruits` and `vegetables` objects.
    fruits = plantCollection.get_fruits()
    vegetables = plantCollection.get_vegetables()
    # create new `fruit` and `vegetable` objects.
    fruit = gdslib.fruitType(
        catalogID="001-002",
        name="Elberta peach",
        description="tasty and juicy",
        plantingInstructions="plant in open sun",
    )
    vegetable = gdslib.vegetableType(
        catalogID="001-002",
        name="Better Boy Tomato",
        description="red and juicy",
        plantingInstructions="plant in cages",
    )
    # add our new `fruit` and `vegetable`.
    fruits.add_fruit(fruit)
    vegetables.add_vegetable(vegetable)
    # write out the updated object tree.
    if opts.outfilename:
        if os.path.exists(opts.outfilename) and not opts.force:
            sys.exit(
                "Error: use option -f/--force to overwrite existing file.")
        else:
            with open(opts.outfilename, 'w') as outfile:
                outfile.write('<?xml version="1.0" encoding="utf-8"?>\n')
                plantCollection.export(outfile, 0, name_=tag)
    else:
        sys.stdout.write('<?xml version="1.0" encoding="utf-8"?>\n')
        plantCollection.export(sys.stdout, 0, name_=tag)


def main():
    description = """\
synopsis:
  parse an XML file; modify an object; write-out the modified object tree.
"""
    epilog = """\

examples:
  python plants_update01.py plants_doc.xml
  python plants_update01.py -v plants_doc.xml
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "infilename",
        help="Input XML file name"
    )
    parser.add_argument(
        "-o", "--outfilename",
        action="store", type=str,
        help="Name/path of file to write to (default is stdout)",
    )
    parser.add_argument(
        "-f", "--force",
        action="store_true",
        help="Force overwrite of output file without asking",
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions.",
    )
    opts = parser.parse_args()
    parse_and_update(opts)


if __name__ == '__main__':
    main()
